/*
 * bungmeter config file
 */

//#define DISPLAY_BITS	1	

#define AVG_RATE		5
#define AREAW			200
#define AREAH			50

#ifdef __linux__
#define BUFSIZE 256
#define PATH_NET_DEV "/proc/net/dev"
static FILE *proc_net_dev;
#endif

char interface[6];

